#@f mdpw.tcl Basic functions for MDPW
#@v alpha1.0 hosoda@cwbp.com 21Oct1998

set VREST -63
#@n :mn make note
#@a p pitch of the note
#@ao d duration of the note
#@ao v velocity of the note
#@ao i instrument of the note
#@o note list
#@g none.
proc :mn {args} {
  set p  [lindex $args 0]
  if {[llength $p] == 0 || $p<0 || $p>127} {
    if {[llength $args] > 2 } {
      set args [lreplace $args 2 2 0]
      set args [lreplace $args 0 0 0]
      puts stdout $args
    } else if {[llength $args] == 2} {
      set arg [:ma $args 0]
    } else {
    } 
  }
  return [concat n $args] 
}
#@n :ma add element at the end of the list
#@a p list
#@a e element
#@o list 
#@g none.
proc :ma {l e} {
  return [linsert $l end $e]
}

#@n :ms make a sequence
#@a l list of lists which denotes notes
#@o sequence
#@g none.
proc :ms {args} {
  return [concat s $args] 
}
#@n :mc make a chord
#@a l list of lists which denote notes
#@o sequence
#@g none.
proc :mc {args} {
  return [concat c $args] 
} 
#@n :pp pick playable or list. 
#@d Make new list from an element list and an index list.
#@d An element list can be playable or not.When it is playable retrun list is playable 
#@d of the same type and the type element in head is igonored in applying index.
#@a p chord, sequence or list
#@a l list of index number
#@o sequence
#@g none.
proc :pp {p l} {
  set t [lindex $p 0]
  if {[lindex $p 0] == "s" || [lindex $p 0] == "c" } {
     set r [list $t]
     set o 1
  } else {
     set r [list]
     set o 0
  }
  foreach i $l {
    set r [:ma $r [lindex $p [expr $i+$o]]]
  }
  return $r
}
#@n :sd 
#@d Set duration to the note list. Skip sequeces and chords.
#@a l playable list.
#@a d duration list .If this is shorter than $l, this list will be repeated.
#@o playable list
#@g none.
proc :sd {l d} {
  set r [list]
  set i 0
  set ld [llength $d]
  foreach e $l {
    if {[lindex $e 0] == "n"} {
      set e [lreplace $e 2 2 [lindex $d $i]]
      incr i
      if {$i >= $ld} {set i 0}  
    }
    set r [:ma $r $e]
  }
  return $r
}
#@n :sv 
#@d Set velocity to the note list.
#@a l playable list.
#@a f velocity from.
#@a t velocity to.
#@o playable list
#@g none.
proc :sv {l f {t 255} } {
#  puts stdout "sv:"+$l
  if {$t > 128} { 
    set t $f 
  }
  if {$l == "s" || $l == "c" } {
    return $l
  }
  set ll [expr [llength $l]-2]
  if { $ll > 0} {
    set s [expr (($t - $f)+0.)/$ll]
  } else {
    set s 0
  }
  set r [list]
  for {set i 0} {$i < [llength $l]} {incr i} {
    set e [lindex $l $i]
    set v [lindex [format %3.0f [expr ($s*($i-1)+$f)]] 0]
    if {[lindex $e 0] == "n"} {
      if {[llength $e] == 3 } {
        set r [:ma $r [:ma $e $v]]
      } elseif {[llength $e] > 3 } {
        set r [:ma $r [lreplace $e 3 3 $v]]
      } else {
        set r [:ma $r $e]
      }
    }  else  {
      set r [:ma $r [:sv $e $v $v]]
    }
  }
  return $r
}
#@n :exp 
#@d expand list.
#@a b base.
#@a l list of pitchs (number).
#@a r times .
#@a o offset.
#@o list of number
proc :exp {b l {r 1} {o 12}} {
 set re [list]
 for {set i 0} {$i < $r} {incr i} {
   foreach li $l {
     set re [linsert $re end [expr $li+$b+$i*$o]]
   }
 }
 return $re
}

#@n :gns. 
#@d generate playable list from  the pitchlist.
#@a l list of pitchs (number).
#@a t type of playable list "s" or "c".
#@a d duration of notes.
#@a v velocity of notes.
#@a i instruments of notes.
#@o playable list
#@g none.
proc :gns {l {t "c"} {d 1} {v 64} {i 1}} {
  set r [list $t]
  foreach e $l {
   set r [:ma $r [:mn $e $d $v $i]]
  }
  return $r
} 
#@n :gnls. 
#@d generate playable list from a list with p, d, v and i.
#@a l list of pitchs (number).
#@a t type of playable list "s" or "c".
#@a d duration of the first note.
#@a v velocity of the first notes.
#@a i instruments of the first notes.
#@o playable list
#@g none.
proc :gnls {l {t "s"} {d 1} {v 64} {i 1}} {
  set r [list $t]
  foreach e $l {
   set p [lindex $e 0]
   if {[llength $e] > 1} {  set d [lindex $e 1]}
   if {[llength $e] > 2} {  set v [lindex $e 2]}
   if {[llength $e] > 3} {  set i [lindex $e 3]}
   set r [:ma $r [:mn $p $d $v $i]]
  }
  return $r
} 
#@n :map 
#@d generate list from  the pitch list and index list.
#@a i list of indexs (number).
#@a l list of pitchs (number).
#@o list of number
#@g none.
proc :map {i l} {
  set r [list]
  foreach e $i {
    set r [:ma $r [lindex $l $e]]
  }
  return $r
}